
.installed-overlay-box {
	font-size: 80%;
	background-color: @theme_selected_bg_color;
	border-radius: 4px;
	color: @theme_selected_fg_color;
	text-shadow: 0 1px 0 rgba(0,0,0,0.5);
}

.installed-overlay-label { }

.index-title-alignment-software {
	font-weight: bold;
	font-size: 14px;
}


GtkNotebook.main-notebook-software > GtkScrolledWindow {
	border-bottom-width: 0px;
	border-left-width: 0px;
	border-right-width: 0px;
}

@keyframes throbbing {
        0% { background-image: linear-gradient(to bottom, transparent, transparent); }
}

.needs-attention {
        background-image: linear-gradient(to bottom, @theme_selected_bg_color, @theme_selected_bg_color);
        animation: throbbing linear 1s infinite alternate;
}

.view.tile {
        border-radius: 0;
        border-width: 3px;
        border-image: url("shadow.png") 3 3 3 3 stretch;
        background-image: none;
        background-color: @list_box_bg;
}

.view.tile:hover {
        background-color: @content_view_bg;
}

.application-details-title {
	font-weight: bold;
	font-size: 14px;
}

.application-details-summary {
}

.application-details-description {
}
